package com.kexun.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.kexun.entity.ReportInfo;
import com.kexun.enums.ReportStatusEnum;
import com.kexun.mapper.ReportInfoMapper;
import com.kexun.model.ro.ReportInfoRO;
import com.kexun.service.ReportInfoService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * <p>
 * 报文信息 服务实现类
 * </p>
 *
 * @author lgs
 * @since 2023-05-10
 */
@Service
@Slf4j
public class ReportInfoServiceImpl extends ServiceImpl<ReportInfoMapper, ReportInfo> implements ReportInfoService {

    @Autowired
    private ReportInfoMapper reportInfoMapper;
    /**
     * 保存报文信息
     * @param reportInfo
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void saveReportInfo(ReportInfo reportInfo){
        super.save(reportInfo);
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateReportInfo(ReportInfoRO reportInfoRo){
        log.info("开始修改报文信息......");


    }

    /**
     * 按条件查询报文信息
     * @param reportInfoRo
     * @return
     */
    @Override
    public List<ReportInfo> listReportInfo(ReportInfoRO reportInfoRo) {
        QueryWrapper queryWrapper = new QueryWrapper();
        if (reportInfoRo.getStatus()!=null){
            queryWrapper.eq("status",reportInfoRo.getStatus());
        }else {
            queryWrapper.eq("status", ReportStatusEnum.NO_PRASE.getValue());
        }
        List<ReportInfo> list = reportInfoMapper.selectList(queryWrapper);
        return list;
    }

    /**
     * 根据主键查询报文信息
     * @param id
     * @return
     */
    public ReportInfo getReportInfo(Long id){
       return reportInfoMapper.selectById(id);
    }


}
