package com.kexun.service.xml;

import com.kexun.constant.BaseConsts;
import com.kexun.entity.IBaseEntity;
import com.kexun.model.dto.CommonParam;
import com.kexun.utils.DateUtils;
import com.kexun.utils.ReflexUtils;
import org.springframework.stereotype.Service;

import java.lang.reflect.Field;
import java.util.Date;
import java.util.Map;

/**
 * Description: 特殊信息的赋值 主要是对公共方法进行赋值
 *
 * @author lgs
 * @date 2023/06/05
 */
@Service
public class CommonSpecService implements IHandleXmlSpecService {
    @Override
    public void handleXmlSpec(CommonParam commonParam, Object jsonBean, IBaseEntity baseEntity) {
        Class sourceClass = jsonBean.getClass();
        Class targetClass = baseEntity.getClass();
        try {
            //1、处理报告日期
            if (ReflexUtils.hasField(sourceClass, BaseConsts.RPT_TIME)) {
                Field sourceField = sourceClass.getDeclaredField(BaseConsts.RPT_TIME);
                sourceField.setAccessible(true);
                //获取值
                String rptTime = String.valueOf(sourceField.get(jsonBean));
                String dateTime = DateUtils.convertTDate(rptTime);
                if (ReflexUtils.hasField(targetClass, BaseConsts.RPT_TIME)) {
                    Field targetField = targetClass.getDeclaredField(BaseConsts.RPT_TIME);
                    String types = targetField.getGenericType().toString();
                    if (types.equals("class java.util.Date")) {
                        Date date = DateUtils.convertStrToDate(dateTime, DateUtils.DATE_YYYY_MM_DD_HH_MM_SS);
                        targetField.setAccessible(true);
                        targetField.set(baseEntity, date);
                    }
                }
            }
            //2、处理报告编码
            copyValue(commonParam, baseEntity, BaseConsts.RPT_NO);
            //3、处理姓名
            copyValue(commonParam, baseEntity, BaseConsts.NAME_);
            //4、证件类型代码
            copyValue(commonParam, baseEntity, BaseConsts.ID_TYPE_CODE);
            //5、证件类型描述
            copyValue(commonParam, baseEntity, BaseConsts.ID_TYPE_DESC);
            //6、证件号码
            copyValue(commonParam, baseEntity, BaseConsts.ID_NUM);
            //7、报文头信息ID
            copyValue(commonParam, baseEntity, BaseConsts.HEAD_INF_ID);
            //8、处理其他信息
            if (commonParam.getNodeMap()!=null){
                Map<String, String> paraMap = commonParam.getNodeMap();
                if (!paraMap.isEmpty()) {
                    //属性名
                    String accType = paraMap.get(BaseConsts.ACC_TYPE_ATTRIBUTE);
                    //属性对应的值
                    String value = paraMap.get(BaseConsts.ACC_TYPE_KEY);
                    handleOtherValue(baseEntity, accType, value);
                }
            }
        } catch (NoSuchFieldException | IllegalAccessException e) {
            e.printStackTrace();

        }
    }

    /**
     * 通过name对源资源和目标资源进行赋值
     *
     * @param commonParam 参数类
     * @param baseEntity  对应的数据库的Entity
     * @param name        字段名
     * @throws Exception 异常信息
     */
    private void copyValue(CommonParam commonParam, IBaseEntity baseEntity, String name) throws NoSuchFieldException, IllegalAccessException {
        Class sourceClass = commonParam.getClass();
        Class targetClass = baseEntity.getClass();
        if (ReflexUtils.hasField(targetClass, name)) {
            Field targetField = targetClass.getDeclaredField(name);
            if (ReflexUtils.hasField(sourceClass, name)) {
                Field sourceField = sourceClass.getDeclaredField(name);
                sourceField.setAccessible(true);
                //获取值
                Object value = String.valueOf(sourceField.get(commonParam));
                String types = targetField.getGenericType().toString();
                if (types.equals("class java.lang.Long")) {
                    value = Long.valueOf(String.valueOf(value));
                }
                targetField.setAccessible(true);
                targetField.set(baseEntity, value);
            }
        }
    }

    /**
     * 处理其他信息
     *
     * @param baseEntity
     * @param name
     * @param value
     * @throws NoSuchFieldException
     * @throws IllegalAccessException
     */
    private void handleOtherValue(IBaseEntity baseEntity, String name, String value) throws NoSuchFieldException, IllegalAccessException {
        Class targetClass = baseEntity.getClass();
        if (ReflexUtils.hasField(targetClass, name)) {
            Field targetField = targetClass.getDeclaredField(name);
            targetField.setAccessible(true);
            targetField.set(baseEntity, value);
        }
    }
}
