package com.kexun.service.xml;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.annotation.JSONField;
import com.kexun.constant.BaseConsts;
import com.kexun.constant.XmlNodeConsts;
import com.kexun.entity.IBaseEntity;
import com.kexun.exception.RRException;
import com.kexun.factory.HandleXmlFactory;
import com.kexun.factory.HandleXmlFactoryBean;
import com.kexun.model.dto.CommonParam;
import com.kexun.model.xml.HeadInfo;
import com.kexun.model.xml.IXmlBase;
import com.kexun.service.CrHeadInfService;
import com.kexun.service.FieldDefineService;
import com.kexun.service.FieldTypeService;
import com.kexun.utils.ParseXmlUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.lang.reflect.Field;
import java.util.*;

/**
 * Description:解析xml处理类
 *
 * @author lgs
 * @date 2023/05/25
 */
@Service
@Slf4j
public class ParseHandleService {

    @Autowired
    HandleXmlFactory handleXmlFactory;
    @Autowired
    CrHeadInfService crHeadInfService;
    @Autowired
    FieldDefineService fieldDefineService;

    /**
     * 解析处理xml的内容
     *
     * @param xmlContent xml报文
     */
    public void parseHandle(String xmlContent) throws Exception {
        //把xml转成Json格式，和XML保持一致的格式
        JSONObject jsonObject = ParseXmlUtils.xmlToJson(xmlContent);
        log.info("转出json后={}", jsonObject.toJSONString());
        /**
         * 处理思路：
         * 1、根据json的key值，获取对应的Json信息段
         * 2、有了key值，识别对应的需要转成的Bean
         * 3、找到对应的service，进行各个表的保存
         */
        //循环json的节点，然后根据json各个节点进行处理
        if (!jsonObject.isEmpty()) {
            //获取HEADBO的数据，需要得到用户的相关信息，循环其他的时候，忽略掉即可
            JSONObject headJsonObject = (JSONObject) jsonObject.get(XmlNodeConsts.HEADBO);
            CommonParam commonParam = new CommonParam();
            handleCommParam(headJsonObject, commonParam);
            //定义Map 存放JSON
            Map<String, Object> jsonMap = new HashMap<>(16);
            for (Map.Entry<String, Object> entry : jsonObject.entrySet()) {
                if (entry.getKey().equals(XmlNodeConsts.HEADBO)) {
                    continue;
                }
                String nodeName = entry.getKey();
                log.info("开始处理节点名称{}的解析工具", nodeName);
                //根据nodeName获取
                HandleXmlFactoryBean factoryBean = getFactoryBean(nodeName);
                if (null != factoryBean) {
                    //得到xml报文信息段对应的JavaBean
                    Class<? extends IXmlBase> xmlModelBean = factoryBean.getModelBean();
                    IXmlBase IXmlBase = xmlModelBean.newInstance();
                    //JSONObject o = (JSONObject) JSONObject.toJSON(entry);
                    String json = ((JSONObject) entry.getValue()).toJSONString();
                    //json转化成JavaBean
                    IXmlBase jsonBean = JSONObject.parseObject(json, IXmlBase.getClass());
                    IHandleXmlBaseService reportXmlBaseService = factoryBean.getReportXmlBaseService();

                    //处理简单类型的
                    if (BaseConsts.NODE_TYPE_NORMAL.equals(factoryBean.getNodeType())) {
                        //赋值
                        Class<? extends IBaseEntity> baseEntity = factoryBean.getBaseEntity();
                        IBaseEntity iBaseEntity = baseEntity.newInstance();
                        BeanUtils.copyProperties(jsonBean, iBaseEntity);
                        if (factoryBean.getSpecService() != null) {
                            Map<String, String> paramMap = factoryBean.getParamMap();
                            commonParam.setNodeMap(paramMap);
                            factoryBean.getSpecService().handleXmlSpec(commonParam, jsonBean, iBaseEntity);
                        }
                        //保存
                        factoryBean.getService().save(iBaseEntity);
                        //构建json
                        if (factoryBean.getBaseFieldPackageJsonService()!=null){
                            factoryBean.getBaseFieldPackageJsonService().packageJson(jsonMap,json,factoryBean.getXmlNodeName(),factoryBean.getNodeType());
                        }
                    } else if (BaseConsts.NODE_TYPE_COMPOSE.equals(factoryBean.getNodeType())) {
                        //处理复杂类型
                        Map<String, String> paramMap = factoryBean.getParamMap();
                        commonParam.setNodeMap(paramMap);
                        reportXmlBaseService.handleXmlSgmt(commonParam, jsonBean);
                        //构建json
                        if (factoryBean.getBaseFieldPackageJsonService()!=null){
                            factoryBean.getBaseFieldPackageJsonService().packageJson(jsonMap,json,factoryBean.getXmlNodeName(),factoryBean.getNodeType());
                        }
                    } else if (BaseConsts.NODE_TYPE_LIST.equals(factoryBean.getNodeType())) {
                        Map<String, String> paramMap = factoryBean.getParamMap();
                        Class<? extends IXmlBase> aClass = jsonBean.getClass();
                        Field[] declaredFields = aClass.getDeclaredFields();
                        for (Field field : declaredFields) {
                            field.setAccessible(true);
                            Class<?> type = field.getType();
                            if (type.equals(List.class)) {
                                String nodeListName = paramMap.get(BaseConsts.NODE_LIST_NAME);
                                //获取注解标记的名字
                                String name = field.getDeclaredAnnotation(JSONField.class).name();
                                if (nodeListName.equals(name)) {
                                    List list = (List) field.get(jsonBean);
                                    List<IBaseEntity> iBaseEntities = new ArrayList<>();
                                    for (Object object : list) {
                                        //object是json转成JavaBean,需要把赋值给Entity,然后保存
                                        Class<? extends IBaseEntity> baseEntity = factoryBean.getBaseEntity();
                                        IBaseEntity iBaseEntity = baseEntity.newInstance();
                                        BeanUtils.copyProperties(object, iBaseEntity);
                                        //处理其他的赋值
                                        if (factoryBean.getSpecService() != null) {
                                            factoryBean.getSpecService().handleXmlSpec(commonParam, jsonBean, iBaseEntity);
                                        }
                                        iBaseEntities.add(iBaseEntity);
                                    }
                                    factoryBean.getService().saveBatch(iBaseEntities);
                                    if (factoryBean.getBaseFieldPackageJsonService()!=null){
                                        factoryBean.getBaseFieldPackageJsonService().packageJson(jsonMap,json,factoryBean.getXmlNodeName(),factoryBean.getNodeType());
                                    }
                                }
                            }
                        }
                    }
                    log.info("处理节点名称{}的解析工具结束", nodeName);
                } else {
                    log.error("获取节点{}对应的处理Bean为空", nodeName);
                }
            }

            log.info(jsonMap.toString());
        } else {
            log.error("xml转Json得到的JSONObject为空");
        }
    }

    /**
     * 处理公共的参数
     *
     * @param headJsonObject
     * @param commonParam
     */
    private void handleCommParam(JSONObject headJsonObject, CommonParam commonParam) {
        //被查询者姓名
        String name = String.valueOf(headJsonObject.get(BaseConsts.NAME));
        //被查询者证件类型代码
        String creTypeCode = String.valueOf(headJsonObject.get(BaseConsts.CRETYPECODE));
        //被查询者证件类型描述
        String creTypeDesc = String.valueOf(headJsonObject.get(BaseConsts.CRETYPEDESC));
        //被查询者证件号码
        String creNumber = String.valueOf(headJsonObject.get(BaseConsts.CRENUMBER));
        //报告编号
        String rptNo = String.valueOf(headJsonObject.get(BaseConsts.RPTNO));
        //构建公共参数
        commonParam.setName(name);
        commonParam.setIdTypeCode(creTypeCode);
        commonParam.setIdTypeDesc(creTypeDesc);
        commonParam.setIdNum(creNumber);
        commonParam.setRptNo(rptNo);
        HeadInfo headInfo = headJsonObject.toJavaObject(HeadInfo.class);

        //返回主键ID
        Long id = crHeadInfService.saveHeadInfo(headInfo);
        commonParam.setHeadInfId(id);
    }

    /**
     * 根据xml节点名称获取相应的处理Bean
     *
     * @param xmlNodeName:xml节点名称
     * @return
     */
    private HandleXmlFactoryBean getFactoryBean(String xmlNodeName) {
        HandleXmlFactoryBean factoryBean = Optional.ofNullable(handleXmlFactory.getFactoryBean(xmlNodeName)).orElse(null);
        return factoryBean;
    }

}
